@@ -8,7 +8,9 @@ from sql.operators import Concat
 from sql.aggregate import Count
 
 from trytond import backend
+from trytond.i18n import gettext
 from trytond.model import Workflow, ModelView, ModelSQL, fields
+from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval, If
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
@@ -277,12 +279,6 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
                     'depends': ['state'],
                     },
                 })
-        cls._error_messages.update({
-                'reset_move': ('You cannot reset to draft move "%s" which was '
-                    'generated by a sale or a purchase.'),
-                'delete_cancel': ('Drop Shipment "%(shipment)s" must be '
-                    'cancelled before deletion.'),
-                })
 
     @staticmethod
     def default_state():
@@ -364,7 +360,9 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
         cls.cancel(shipments)
         for shipment in shipments:
             if shipment.state != 'cancel':
-                cls.raise_user_error('delete_cancel', {
+                raise AccessError(
+                    gettext('sale_supply_drop_shipment'
+                        '.msg_drop_shipment_delete_cancel') % {
                         'shipment': shipment.rec_name,
                         })
         Move.delete([m for s in shipments for m in s.supplier_moves])
@@ -395,7 +393,9 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
             for move in shipment.moves:
                 if (move.state == 'cancel'
                         and isinstance(move.origin, (PurchaseLine, SaleLine))):
-                    cls.raise_user_error('reset_move', (move.rec_name,))
+                    raise AccessError(
+                        gettext('sale_supply_drop_shipment.msg_reset_move',
+                            move=move.rec_name))
         Move.draft([m for s in shipments for m in s.moves
                 if m.state != 'staging'])
 
@@ -404,13 +404,12 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
         pool = Pool()
         UoM = pool.get('product.uom')
         Move = pool.get('stock.move')
-        Currency = pool.get('currency.currency')
 
         to_save = []
         cost_exp = Decimal(str(10.0 ** -Move.cost_price.digits[1]))
         for shipment in shipments:
-            product_qty = defaultdict(lambda: 0)
-            product_cost = defaultdict(lambda: 0)
+            product_qty = defaultdict(int)
+            product_cost = defaultdict(int)
 
             for c_move in shipment.customer_moves:
                 if c_move.state == 'cancel':
@@ -419,19 +418,14 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
                     c_move.uom, c_move.quantity, c_move.product.default_uom,
                     round=False)
 
-            s_product_qty = defaultdict(lambda: 0)
+            s_product_qty = defaultdict(int)
             for s_move in shipment.supplier_moves:
                 if s_move.state == 'cancel':
                     continue
-                internal_quantity = Decimal(str(s_move.internal_quantity))
-                with Transaction().set_context(date=s_move.effective_date):
-                    unit_price = Currency.compute(
-                        s_move.currency, s_move.unit_price,
-                        s_move.company.currency, round=False)
-                unit_price = UoM.compute_price(
-                    s_move.uom, unit_price, s_move.product.default_uom)
-                product_cost[s_move.product] += (
-                    unit_price * internal_quantity)
+                if s_move.cost_price:
+                    internal_quantity = Decimal(str(s_move.internal_quantity))
+                    product_cost[s_move.product] += (
+                        s_move.unit_price * internal_quantity)
 
                 quantity = UoM.compute_qty(
                     s_move.uom, s_move.quantity, s_move.product.default_uom,
@@ -517,37 +511,32 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
             requests += PurchaseRequest.search([
                     ('purchase_line', 'in', list(sub_lines)),
                     ])
-        pline2requests = defaultdict(list)
-        for request in requests:
-            pline2requests[request.purchase_line].append(request)
+        pline2request = {r.purchase_line: r for r in requests}
         sale_lines = SaleLine.search([
                 ('purchase_request', 'in', [r.id for r in requests]),
                 ])
-        request2slines = defaultdict(list)
-        for sale_line in sale_lines:
-            request2slines[sale_line.purchase_request].append(sale_line)
+        request2sline = {sl.purchase_request: sl for sl in sale_lines}
 
         to_save = []
         for shipment in shipments:
             for move in shipment.supplier_moves:
                 if not move.origin:
                     continue
-                for request in pline2requests[move.origin]:
-                    for sale_line in request2slines[request]:
-                        for move in sale_line.moves:
-                            if (move.state not in ('cancel', 'done')
-                                    and not move.shipment
-                                    and move.from_location.type == 'drop'):
-                                move.shipment = shipment
-                                to_save.append(move)
+                sale_line = request2sline.get(pline2request.get(move.origin))
+                if not sale_line:
+                    continue
+                for move in sale_line.moves:
+                    if (move.state not in ('cancel', 'done')
+                            and not move.shipment
+                            and move.from_location.type == 'drop'):
+                        move.shipment = shipment
+                        to_save.append(move)
         Move.save(to_save)
-        Move.draft(to_save)
         cls._synchronize_moves(shipments)
 
     @classmethod
     @ModelView.button
     @Workflow.transition('shipped')
-    @process_sale('customer_moves')
     @process_purchase('supplier_moves')
     def ship(cls, shipments):
         pool = Pool()
@@ -590,8 +579,7 @@ class Move(metaclass=PoolMeta):
         if isinstance(self.origin, SaleLine):
             return self.origin.sale.party.id
         elif isinstance(self.origin, PurchaseLine):
-            if self.origin.purchase.customer:
-                return self.origin.purchase.customer.id
+            return self.origin.purchase.customer.id
 
     @classmethod
     def search_customer_drop(cls, name, clause):
